/* This file is part of smf.
   Copyright (C) 2006, 2007, 2010 Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <signal.h>
#include <syslog.h>
#include <grp.h>
#include <pwd.h>
#include <ctype.h>
#include <sysexits.h> /* FIXME */
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <wordsplit.h>
#include <ostr.h>
#include <kwtab.h>
#include <srvman.h>

#define gettext(s) s
#define _(s) s
#define N_(s) s
   
#if defined HAVE_SYSCONF && defined _SC_OPEN_MAX
# define getmaxfd() sysconf(_SC_OPEN_MAX)
#elif defined (HAVE_GETDTABLESIZE)
# define getmaxfd() getdtablesize()
#else
# define getmaxfd() 256
#endif

typedef union {
	struct sockaddr sa;
	struct sockaddr_in s_in;
	struct sockaddr_un s_un;
} all_addr_t;

/* mem.c */
void emalloc_die(size_t size);
void *emalloc(size_t size);
void *ecalloc(size_t nmemb, size_t size);
void *erealloc(void *ptr, size_t newsize);
char *estrdup(const char *s);

/* url.c */
int parse_url(const char *cstr, struct sockaddr **psa, socklen_t *plen);

/* log.c */
extern char *progname;
extern char *log_tag;
extern int log_facility;
extern int log_to_stderr;
extern int smap_trace_option;

extern smap_ostream_t smap_error_str;
extern smap_ostream_t smap_debug_str;
extern smap_ostream_t smap_trace_str;

void smap_verror(const char *fmt, va_list ap);
void smap_error(const char *fmt, ...);
void set_program_name(char *name);
void smap_log_init(void);

int syslog_fname_to_n (const char *str, int *pint);
void debug_printf(const char *fmt, ...);

#define DBG_SMAP 0
#define DBG_SRVMAN 1
#define MAX_DEBUG_COUNT 2
extern int debug_level[];

#define debug_lev_p(num, lev)			\
	(smap_debug_str && debug_level[num] > lev)

#define debug(num, lev, args) do {		\
	if (debug_lev_p(num, lev))	        \
		debug_printf args;		\
    } while (0)

int debug_set(const char *spec);

/* smapostr.c */
smap_ostream_t smap_map_ostream_create(FILE *fp);

/* smapd. */
extern int foreground;
extern unsigned smap_timeout;
extern int inetd_mode;
extern int lint_mode;
extern char *pidfile;

/* cfg.c */
extern const char *cfg_file_name;
extern unsigned cfg_line;

enum kw_type {
	KWT_STRING,
	KWT_BOOL,
	KWT_INT,
	KWT_INT_O,
	KWT_INT_X,
	KWT_UINT,
	KWT_UINT_O,
	KWT_UINT_X,
	KWT_ARR,
	KWT_FUN
};
	
struct cfg_kw {
	const char *kw;
	enum kw_type type;
	int *ival;
	char **sval;
	char ***aval;
	int (*fun)(struct cfg_kw *, int, char **);
};

int parse_config(const char *filename, struct cfg_kw *);

int cfg_chkargc(int wordc, int min, int max);
int cfg_parse_bool(const char *str, int *res);
